@@ -63,6 +63,84 @@ static int do_old_idr(int, ulong, struct list_pair *);
 #define PROG_VERBOSE  (0x40)
 #define MAP_VERBOSE   (0x80)
 
+static int map_is_per_cpu(int type)
+{
+
+/* See the definition of bpf_map_type: include/uapi/linux/bpf.h */
+#define BPF_MAP_TYPE_PERCPU_HASH (5UL)
+#define BPF_MAP_TYPE_PERCPU_ARRAY (6UL)
+#define BPF_MAP_TYPE_LRU_PERCPU_HASH (10UL)
+#define BPF_MAP_TYPE_PERCPU_CGROUP_STORAGE (21UL)
+
+	return type == BPF_MAP_TYPE_PERCPU_HASH ||
+	       type == BPF_MAP_TYPE_PERCPU_ARRAY ||
+	       type == BPF_MAP_TYPE_LRU_PERCPU_HASH ||
+	       type == BPF_MAP_TYPE_PERCPU_CGROUP_STORAGE;
+}
+
+static int map_is_fd_map(int type)
+{
+
+/* See the definition of bpf_map_type: include/uapi/linux/bpf.h */
+#define BPF_MAP_TYPE_PROG_ARRAY (3UL)
+#define BPF_MAP_TYPE_PERF_EVENT_ARRAY (4UL)
+#define BPF_MAP_TYPE_CGROUP_ARRAY (8UL)
+#define BPF_MAP_TYPE_ARRAY_OF_MAPS (12UL)
+#define BPF_MAP_TYPE_HASH_OF_MAPS (13UL)
+
+	return type == BPF_MAP_TYPE_PROG_ARRAY ||
+	       type == BPF_MAP_TYPE_PERF_EVENT_ARRAY ||
+	       type == BPF_MAP_TYPE_CGROUP_ARRAY ||
+	       type == BPF_MAP_TYPE_ARRAY_OF_MAPS ||
+	       type == BPF_MAP_TYPE_HASH_OF_MAPS;
+
+}
+
+static ulong bpf_map_memory_size(int map_type, uint value_size,
+				uint key_size, uint max_entries)
+{
+	ulong size;
+	uint valsize;
+	int cpus = 0;
+
+	if (map_is_per_cpu(map_type)) {
+		cpus = get_cpus_possible();
+		if (!cpus) {
+			error(WARNING, "cpu_possible_map does not exist, possible cpus: %d\n", cpus);
+			return 0;
+		}
+
+		valsize = roundup(value_size, 8) * cpus;
+	} else if (map_is_fd_map(map_type))
+		valsize = sizeof(uint);
+	else
+		valsize = value_size;
+
+	size = roundup((key_size + valsize), 8);
+
+	return roundup((max_entries * size), PAGESIZE());
+}
+
+/*
+ * Code replicated from kernel/bpf/ringbuf.c:ringbuf_map_mem_usage()
+*/
+static ulong ringbuf_map_mem_usage(struct bpf_info *bpf, int i)
+{
+	ulong nr_pages;
+	ulong usage = SIZE(bpf_ringbuf_map);
+	ulong rb = (ulong)(bpf->maplist[i].value) - OFFSET(bpf_ringbuf_map_map)
+			+ OFFSET(bpf_ringbuf_map_rb);
+	readmem(rb, KVADDR, &rb, sizeof(ulong), "bpf_ringbuf *", FAULT_ON_ERROR);
+	readmem(rb + OFFSET(bpf_ringbuf_nr_pages), KVADDR, &nr_pages,
+		sizeof(ulong), "bpf_ringbuf.nr_pages", FAULT_ON_ERROR);
+	usage += (nr_pages << PAGESHIFT());
+	ulong nr_meta_pages = (OFFSET(bpf_ringbuf_consumer_pos) >> PAGESHIFT()) + 2;
+	ulong nr_data_pages = UINT(bpf->bpf_map_buf + OFFSET(bpf_map_max_entries)) >> PAGESHIFT();
+	usage += (nr_meta_pages + 2 * nr_data_pages) * sizeof(void *);
+
+	return usage;
+}
+
 void
 cmd_bpf(void)
 {
@@ -159,6 +237,7 @@ bpf_init(struct bpf_info *bpf)
 		STRUCT_SIZE_INIT(bpf_prog_aux, "bpf_prog_aux");
 		STRUCT_SIZE_INIT(bpf_map, "bpf_map");
 		STRUCT_SIZE_INIT(bpf_insn, "bpf_insn");
+		STRUCT_SIZE_INIT(bpf_ringbuf_map, "bpf_ringbuf_map");
 		MEMBER_OFFSET_INIT(bpf_prog_aux, "bpf_prog", "aux");
 		MEMBER_OFFSET_INIT(bpf_prog_type, "bpf_prog", "type");
 		MEMBER_OFFSET_INIT(bpf_prog_tag, "bpf_prog", "tag");
@@ -170,6 +249,10 @@ bpf_init(struct bpf_info *bpf)
 		MEMBER_OFFSET_INIT(bpf_map_map_flags, "bpf_map", "map_flags");
 		MEMBER_OFFSET_INIT(bpf_prog_aux_used_maps, "bpf_prog_aux", "used_maps");
 		MEMBER_OFFSET_INIT(bpf_prog_aux_used_map_cnt, "bpf_prog_aux", "used_map_cnt");
+		MEMBER_OFFSET_INIT(bpf_ringbuf_map_map, "bpf_ringbuf_map", "map");
+		MEMBER_OFFSET_INIT(bpf_ringbuf_map_rb, "bpf_ringbuf_map", "rb");
+		MEMBER_OFFSET_INIT(bpf_ringbuf_consumer_pos, "bpf_ringbuf", "consumer_pos");
+		MEMBER_OFFSET_INIT(bpf_ringbuf_nr_pages, "bpf_ringbuf", "nr_pages");
 		if (!VALID_STRUCT(bpf_prog) || 
 		    !VALID_STRUCT(bpf_prog_aux) ||
 		    !VALID_STRUCT(bpf_map) ||
@@ -332,7 +415,7 @@ do_bpf(ulong flags, ulong prog_id, ulong map_id, int radix)
 {
 	struct bpf_info *bpf;
 	int i, c, found, entries, type;
-	uint uid, map_pages, key_size, value_size, max_entries;
+	uint uid, map_pages, key_size = 0, value_size = 0, max_entries = 0;
 	ulong bpf_prog_aux, bpf_func, end_func, addr, insnsi, user;
 	ulong do_progs, do_maps;
 	ulonglong load_time;
@@ -572,6 +655,8 @@ do_map_only:
 		fprintf(fp, "\n");
 
 		if (flags & (MAP_ID|MAP_VERBOSE)) {
+			ulong msize = 0;
+
 			fprintf(fp, "     KEY_SIZE: ");
 			if (VALID_MEMBER(bpf_map_key_size)) {
 				key_size = UINT(bpf->bpf_map_buf + OFFSET(bpf_map_key_size));
@@ -602,8 +687,13 @@ do_map_only:
 			} else if (VALID_MEMBER(bpf_map_pages)) {
 				map_pages = UINT(bpf->bpf_map_buf + OFFSET(bpf_map_pages));
 				fprintf(fp, "%d\n", map_pages * PAGESIZE());
+			} else if ((msize = bpf_map_memory_size(type, value_size, key_size, max_entries)))
+				fprintf(fp, "%ld\n", msize);
+#define BPF_MAP_TYPE_RINGBUF (0x1BUL)
+			else if (type == BPF_MAP_TYPE_RINGBUF) {
+				fprintf(fp, "%ld\n", ringbuf_map_mem_usage(bpf, i));
 			} else
-				fprintf(fp, "(unknown)\n");
+				fprintf(fp, "(unknown)");
 
 			fprintf(fp, "     NAME: ");
 			if (VALID_MEMBER(bpf_map_name)) {
@@ -632,7 +722,7 @@ do_map_only:
 				else
 					fprintf(fp, "(unknown)\n");
 			} else
-				fprintf(fp, "(unknown)\n");
+				fprintf(fp, "(unused)\n");
 		}
 
 		if (flags & DUMP_STRUCT) {
