@@ -169,7 +169,9 @@ void add_extra_lib(char *);
 #define TARGET_CFLAGS_LOONGARCH64     "TARGET_CFLAGS="
 #define TARGET_CFLAGS_LOONGARCH64_ON_X86_64	"TARGET_CFLAGS="
 
+#ifndef GDB_TARGET_DEFAULT
 #define GDB_TARGET_DEFAULT        "GDB_CONF_FLAGS="
+#endif
 #define GDB_TARGET_ARM_ON_X86     "GDB_CONF_FLAGS=--target=arm-elf-linux"
 #define GDB_TARGET_ARM_ON_X86_64  "GDB_CONF_FLAGS=--target=arm-elf-linux CFLAGS=-m32 CXXFLAGS=-m32"
 #define GDB_TARGET_X86_ON_X86_64  "GDB_CONF_FLAGS=--target=i686-pc-linux-gnu CFLAGS=-m32 CXXFLAGS=-m32"
@@ -195,9 +197,10 @@ void add_extra_lib(char *);
 #define GDB_7_3_1 (4)
 #define GDB_7_6   (5)
 #define GDB_10_2   (6)
-#define SUPPORTED_GDB_VERSIONS (GDB_10_2 + 1)
+#define GDB_16_2   (7)
+#define SUPPORTED_GDB_VERSIONS (GDB_16_2 + 1)
 
-int default_gdb = GDB_10_2;
+int default_gdb = GDB_16_2;
 
 struct supported_gdb_version {
 	char *GDB;
@@ -271,6 +274,15 @@ struct supported_gdb_version {
             "GDB_FLAGS=-DGDB_10_2",
             "GPLv3"
         },
+        {
+            "GDB=gdb-16.2",
+            "16.2",
+            "GDB_FILES=${GDB_16.2_FILES}",
+            "GDB_OFILES=${GDB_16.2_OFILES}",
+            "GDB_PATCH_FILES=gdb-16.2.patch",
+            "GDB_FLAGS=-DGDB_16_2",
+            "GPLv3"
+        },
 };
 
 #define DAEMON  0x1
@@ -376,6 +388,7 @@ get_current_configuration(struct supported_gdb_version *sp)
 	static char buf[512];
 	char *p;
 
+#ifndef CONF_TARGET_ARCH
 #ifdef __alpha__
         target_data.target = ALPHA;
 #endif
@@ -422,6 +435,9 @@ get_current_configuration(struct supported_gdb_version *sp)
 #ifdef __loongarch64
 	target_data.target = LOONGARCH64;
 #endif
+#else
+       target_data.target = CONF_TARGET_ARCH;
+#endif
 
 	set_initial_target(sp);
 
@@ -1598,7 +1614,12 @@ setup_gdb_defaults(void)
                         fprintf(stderr, ".gdb configuration: %s\n", sp->GDB_VERSION_IN);
                         return store_gdb_defaults(sp);
                 }
-
+                if (strcmp(buf, "16.2") == 0) {
+                        fclose(fp);
+                        sp = &supported_gdb_versions[GDB_16_2];
+                        fprintf(stderr, ".gdb configuration: %s\n", sp->GDB_VERSION_IN);
+                        return store_gdb_defaults(sp);
+                }
         }
 	
 	fclose(fp);
